/**
 * @file
 * Applies tabs on views semantic tabs display.
 */

(function (Drupal) {
  Drupal.behaviors.viewsSemanticTabs = {
    attach: function (context, settings) {
      const $activeTabClassName = 'selected';
      const $activePanelClassName = 'current';
      const $disabledPanelClassName = 'hidden';

      once(
        'viewsSemanticTabs',
        '.views-semantic-tabs',
        context
      ).forEach($tabsWrapper => {

        // Gets all links inside tab wrapper.
        const $tabs = $tabsWrapper.querySelectorAll('ul.tabs-list > li > a');
        if ($tabs === null) return;

        const $panels = $tabsWrapper.querySelectorAll('.tab-panel');
        const getCurrentTabIndex = () => [...$tabs].findIndex(($checkTab) => $checkTab.classList.contains($activeTabClassName));

        $tabs.forEach($tab => {

          // Click event listener.
          $tab.addEventListener('click', function(ev) {
            const $tabSiblings = [...$tabs].filter(($checkTab) => $checkTab !== $tab);

            // Disable sibling tabs:
            $tabSiblings.forEach($tabSibling => {
              $tabSibling.setAttribute('aria-selected', false);
              $tabSibling.setAttribute('tabindex', -1);
              $tabSibling.classList.remove($activeTabClassName);
            });

            // Activate current tab.
            $tab.focus();
            $tab.setAttribute('aria-selected', true);
            $tab.setAttribute('tabindex', 0);
            $tab.classList.add($activeTabClassName);

            // Get the target component
            const $targetPanelId = $tab.getAttribute('aria-controls');
            const $targetPanel = document.getElementById($targetPanelId);
            const $panelSiblings = [...$panels].filter(($checkPanel) => $checkPanel !== $targetPanel);

            // Disable sibling panels
            $panelSiblings.forEach($panelSibling => {
              $panelSibling.classList.remove($activePanelClassName);
              $panelSibling.classList.add($disabledPanelClassName);
            })

            // Activate selected panel.
            $targetPanel.classList.remove($disabledPanelClassName);
            $targetPanel.classList.add($activePanelClassName);

            ev.preventDefault();
          });

          // Keydown event listener.
          $tab.addEventListener('keydown', (ev) => {
            const $leftArrow = 37;
            const $upArrow = 38;
            const $rightArrow = 39;
            const $downArrow = 40;
            const $key = ev.which || ev.keyCode;

            // if the key pressed was an arrow...
            if ($key >= $leftArrow && $key <= $downArrow){

              const getUpdatedIndex = () => {
                const $index = getCurrentTabIndex();
                const $highestTabId = $tabs.length - 1;

                // move left one tab for left and up arrows.
                if ($key == $leftArrow || $key == $upArrow){
                  if ($index > 0) {
                    return $index - 1;
                  }

                  // If active tab is the first, jump to the last.
                  else {
                    return $highestTabId;
                  }
                }

                // Move right one tab for right and down arrows.
                else {
                  if ($index < $highestTabId){
                    return $index + 1;
                  }

                  // If active tab is the last, jump to the first.
                  else {
                    return 0;
                  }
                } 
              }

              // trigger a click event on the tab to move to
              $tabs.item(getUpdatedIndex()).dispatchEvent(new Event("click"));
              ev.preventDefault();
            }
          });
        });
      });
    }
  };
})(Drupal);
